@@ -20,7 +20,7 @@
 getWebServiceData <- function(obs_url){
   possibleError <- tryCatch({
     h <- basicHeaderGatherer()
-    returnedDoc <- getURI(obs_url, headerfunction = h$update)      
+    returnedDoc <- getURI(obs_url, headerfunction = h$update, encoding='gzip')      
   }, warning = function(w) {
     warning(w, "with url:", obs_url)
   }, error = function(e) {
